@@ -1,13 +1,9 @@
+from __future__ import annotations
+
 import datetime
 import re
 import string
 
-from typing import List
-from typing import Optional
-from typing import Tuple
-from typing import Type
-from typing import Union
-
 from tomlkit._compat import decode
 from tomlkit._utils import RFC_3339_LOOSE
 from tomlkit._utils import _escaped
@@ -64,11 +60,11 @@ class Parser:
     Parser for TOML documents.
     """
 
-    def __init__(self, string: str) -> None:
+    def __init__(self, string: str | bytes) -> None:
         # Input to parse
         self._src = Source(decode(string))
 
-        self._aot_stack: List[Key] = []
+        self._aot_stack: list[Key] = []
 
     @property
     def _state(self):
@@ -92,14 +88,14 @@ class Parser:
         """
         return self._src.extract()
 
-    def inc(self, exception: Optional[Type[ParseError]] = None) -> bool:
+    def inc(self, exception: type[ParseError] | None = None) -> bool:
         """
         Increments the parser if the end of the input has not been reached.
         Returns whether or not it was able to advance.
         """
         return self._src.inc(exception=exception)
 
-    def inc_n(self, n: int, exception: Optional[Type[ParseError]] = None) -> bool:
+    def inc_n(self, n: int, exception: type[ParseError] | None = None) -> bool:
         """
         Increments the parser by n characters
         if the end of the input has not been reached.
@@ -205,7 +201,7 @@ class Parser:
 
         return parent_parts == child_parts[: len(parent_parts)]
 
-    def _parse_item(self) -> Optional[Tuple[Optional[Key], Item]]:
+    def _parse_item(self) -> tuple[Key | None, Item] | None:
         """
         Attempts to parse the next item and returns it, along with its key
         if the item is value-like.
@@ -241,7 +237,7 @@ class Parser:
 
         return self._parse_key_value(True)
 
-    def _parse_comment_trail(self, parse_trail: bool = True) -> Tuple[str, str, str]:
+    def _parse_comment_trail(self, parse_trail: bool = True) -> tuple[str, str, str]:
         """
         Returns (comment_ws, comment, trail)
         If there is no comment, comment_ws and comment will
@@ -302,7 +298,7 @@ class Parser:
 
         return comment_ws, comment, trail
 
-    def _parse_key_value(self, parse_comment: bool = False) -> Tuple[Key, Item]:
+    def _parse_key_value(self, parse_comment: bool = False) -> tuple[Key, Item]:
         # Leading indent
         self.mark()
 
@@ -497,11 +493,12 @@ class Parser:
                             pass
 
                         time_raw = self.extract()
-                        if not time_raw.strip():
-                            trivia.comment_ws = time_raw
+                        time_part = time_raw.rstrip()
+                        trivia.comment_ws = time_raw[len(time_part) :]
+                        if not time_part:
                             return date
 
-                        dt = parse_rfc3339(raw + time_raw)
+                        dt = parse_rfc3339(raw + time_part)
                         assert isinstance(dt, datetime.datetime)
                         return DateTime(
                             dt.year,
@@ -513,7 +510,7 @@ class Parser:
                             dt.microsecond,
                             dt.tzinfo,
                             trivia,
-                            raw + time_raw,
+                            raw + time_part,
                         )
                     except ValueError:
                         raise self.parse_error(InvalidDateError)
@@ -563,7 +560,7 @@ class Parser:
         # Consume opening bracket, EOF here is an issue (middle of array)
         self.inc(exception=UnexpectedEofError)
 
-        elems: List[Item] = []
+        elems: list[Item] = []
         prev_value = None
         while True:
             # consume whitespace
@@ -670,7 +667,7 @@ class Parser:
 
         return InlineTable(elems, Trivia())
 
-    def _parse_number(self, raw: str, trivia: Trivia) -> Optional[Item]:
+    def _parse_number(self, raw: str, trivia: Trivia) -> Item | None:
         # Leading zeros are not allowed
         sign = ""
         if raw.startswith(("+", "-")):
@@ -802,9 +799,17 @@ class Parser:
         value = ""
 
         # A newline immediately following the opening delimiter will be trimmed.
-        if delim.is_multiline() and self._current == "\n":
-            # consume the newline, EOF here is an issue (middle of string)
-            self.inc(exception=UnexpectedEofError)
+        if delim.is_multiline():
+            if self._current == "\n":
+                # consume the newline, EOF here is an issue (middle of string)
+                self.inc(exception=UnexpectedEofError)
+            else:
+                cur = self._current
+                with self._state(restore=True):
+                    if self.inc():
+                        cur += self._current
+                if cur == "\r\n":
+                    self.inc_n(2, exception=UnexpectedEofError)
 
         escaped = False  # whether the previous key was ESCAPE
         while True:
@@ -880,8 +885,8 @@ class Parser:
                 self.inc(exception=UnexpectedEofError)
 
     def _parse_table(
-        self, parent_name: Optional[Key] = None, parent: Optional[Table] = None
-    ) -> Tuple[Key, Union[Table, AoT]]:
+        self, parent_name: Key | None = None, parent: Table | None = None
+    ) -> tuple[Key, Table | AoT]:
         """
         Parses a table element.
         """
@@ -910,8 +915,6 @@ class Parser:
             raise self.parse_error(UnexpectedEofError)
         elif self._current != "]":
             raise self.parse_error(UnexpectedCharError, self._current)
-        elif not key.key.strip():
-            raise self.parse_error(EmptyTableNameError)
 
         key.sep = ""
         full_key = key
@@ -948,6 +951,7 @@ class Parser:
             is_aot,
             name=name_parts[0].key if name_parts else key.key,
             display_name=full_key.as_string(),
+            is_super_table=False,
         )
 
         if len(name_parts) > 1:
@@ -959,7 +963,7 @@ class Parser:
                 # So we have to create the parent tables
                 table = Table(
                     Container(True),
-                    Trivia(indent, cws, comment, trail),
+                    Trivia("", cws, comment, trail),
                     is_aot and name_parts[0] in self._aot_stack,
                     is_super_table=True,
                     name=name_parts[0].key,
@@ -1035,7 +1039,7 @@ class Parser:
 
         return key, result
 
-    def _peek_table(self) -> Tuple[bool, Key]:
+    def _peek_table(self) -> tuple[bool, Key]:
         """
         Peeks ahead non-intrusively by cloning then restoring the
         initial state of the parser.
@@ -1099,7 +1103,7 @@ class Parser:
                 break
             return buf
 
-    def _peek_unicode(self, is_long: bool) -> Tuple[Optional[str], Optional[str]]:
+    def _peek_unicode(self, is_long: bool) -> tuple[str | None, str | None]:
         """
         Peeks ahead non-intrusively by cloning then restoring the
         initial state of the parser.
